import unittest

import numpy as np

import celiagg as agg


@unittest.skipIf(agg.HAS_TEXT, 'Text support is available')
class TestNoTextSupport(unittest.TestCase):
    def test_no_text_font_failure(self):
        with self.assertRaises(RuntimeError):
            agg.Font()

    def test_no_text_draw_text_failure(self):
        buffer = np.zeros((1, 1), dtype=np.uint8)
        canvas = agg.CanvasG8(buffer)
        transform = agg.Transform()
        line_paint = agg.SolidPaint(1.0, 1.0, 1.0)
        gs = agg.GraphicsState()

        with self.assertRaises(RuntimeError):
            canvas.draw_text('', None, transform, line_paint, line_paint, gs)



import unittest

import numpy as np

from celiagg import (GraphicsState, BlendMode, DrawingMode, Image, InnerJoin,
                     LineCap, LineJoin, PixelFormat, Rect, TextDrawingMode)


def array_bases_equal(arr0, arr1):
    return (arr0.__array_interface__['data'][0] ==
            arr1.__array_interface__['data'][0])


class TestGraphicsState(unittest.TestCase):
    def test_rect(self):
        r = Rect(0, 0, -1, -1)
        self.assertFalse(r.valid)

        r = Rect(2.0, 3.0, 5.0, 6.0)
        self.assertEqual(r.x, 2.0)
        self.assertEqual(r.y, 3.0)
        self.assertEqual(r.w, 5.0)
        self.assertEqual(r.h, 6.0)

        gs = GraphicsState()
        self.assertFalse(gs.clip_box.valid)

    def test_state_bad_value_types(self):
        gs = GraphicsState()

        with self.assertRaises(TypeError):
            gs.clip_box = (0, 1, 2, 3)
        with self.assertRaises(TypeError):
            gs.clip_box = None

        with self.assertRaises(TypeError):
            gs.stencil = 'dur hur'

    def test_state_properties(self):
        gs = GraphicsState()

        gs.anti_aliased = True
        self.assertTrue(gs.anti_aliased)
        gs.anti_aliased = False
        self.assertFalse(gs.anti_aliased)

        gs.drawing_mode = DrawingMode.DrawEofFill
        self.assertEqual(gs.drawing_mode, DrawingMode.DrawEofFill)
        gs.drawing_mode = DrawingMode.DrawFillStroke
        self.assertEqual(gs.drawing_mode, DrawingMode.DrawFillStroke)

        gs.text_drawing_mode = TextDrawingMode.TextDrawStroke
        self.assertEqual(gs.text_drawing_mode,
                         TextDrawingMode.TextDrawStroke)
        gs.text_drawing_mode = TextDrawingMode.TextDrawFillStroke
        self.assertEqual(gs.text_drawing_mode,
                         TextDrawingMode.TextDrawFillStroke)

        gs.blend_mode = BlendMode.BlendXor
        self.assertEqual(gs.blend_mode, BlendMode.BlendXor)
        gs.blend_mode = BlendMode.BlendLighten
        self.assertEqual(gs.blend_mode, BlendMode.BlendLighten)

        gs.image_blend_mode = BlendMode.BlendXor
        self.assertEqual(gs.image_blend_mode, BlendMode.BlendXor)
        gs.image_blend_mode = BlendMode.BlendLighten
        self.assertEqual(gs.image_blend_mode, BlendMode.BlendLighten)

        gs.line_cap = LineCap.CapSquare
        self.assertEqual(gs.line_cap, LineCap.CapSquare)
        gs.line_cap = LineCap.CapRound
        self.assertEqual(gs.line_cap, LineCap.CapRound)

        gs.line_join = LineJoin.JoinRound
        self.assertEqual(gs.line_join, LineJoin.JoinRound)
        gs.line_join = LineJoin.JoinBevel
        self.assertEqual(gs.line_join, LineJoin.JoinBevel)

        gs.inner_join = InnerJoin.InnerRound
        self.assertEqual(gs.inner_join, InnerJoin.InnerRound)
        gs.inner_join = InnerJoin.InnerBevel
        self.assertEqual(gs.inner_join, InnerJoin.InnerBevel)

        gs.miter_limit = 3.14
        self.assertEqual(gs.miter_limit, 3.14)
        gs.miter_limit = 0.5
        self.assertEqual(gs.miter_limit, 0.5)

        gs.inner_miter_limit = 3.14
        self.assertEqual(gs.inner_miter_limit, 3.14)
        gs.inner_miter_limit = 0.5
        self.assertEqual(gs.inner_miter_limit, 0.5)

        gs.master_alpha = 0.42
        self.assertEqual(gs.master_alpha, 0.42)

        gs.line_width = 10.0
        self.assertEqual(gs.line_width, 10.0)

        box = Rect(0.0, 0.0, 10.0, 20.0)
        gs.clip_box = box
        self.assertEqual(gs.clip_box, box)
        self.assertIsNot(gs.clip_box, box)

        dashes = [(1.0, 2.0), (3.0, 4.0)]
        gs.line_dash_pattern = dashes
        gs.line_dash_phase = 5.0
        self.assertEqual(gs.line_dash_pattern, dashes)
        self.assertEqual(gs.line_dash_phase, 5.0)
        gs.line_dash_pattern = []
        self.assertEqual(len(gs.line_dash_pattern), 0)

        img = Image(np.zeros((10, 10), dtype=np.uint8), PixelFormat.Gray8)
        self.assertIsNone(gs.stencil)
        gs.stencil = img
        self.assertIs(gs.stencil, img)
        gs.stencil = None
        self.assertIsNone(gs.stencil)

        img = Image(np.zeros((10, 10, 3), dtype=np.uint8), PixelFormat.RGB24)
        with self.assertRaises(ValueError):
            gs.stencil = img

    def test_kwargs_initialization(self):
        box = Rect(0.0, 0.0, 10.0, 20.0)
        dashes = [(1.0, 2.0), (3.0, 4.0)]
        img = Image(np.zeros((10, 10), dtype=np.uint8), PixelFormat.Gray8)
        gs = GraphicsState(
            anti_aliased=True,
            drawing_mode=DrawingMode.DrawEofFill,
            text_drawing_mode=TextDrawingMode.TextDrawFillStroke,
            blend_mode=BlendMode.BlendXor,
            image_blend_mode=BlendMode.BlendXor,
            line_cap=LineCap.CapSquare,
            line_join=LineJoin.JoinRound,
            inner_join=InnerJoin.InnerRound,
            miter_limit=3.14,
            inner_miter_limit=3.14,
            master_alpha=0.42,
            line_width=10.0,
            clip_box=box,
            line_dash_pattern=dashes,
            line_dash_phase=3.5,
            stencil=img
        )

        self.assertTrue(gs.anti_aliased)
        self.assertEqual(gs.drawing_mode, DrawingMode.DrawEofFill)
        self.assertEqual(gs.text_drawing_mode,
                         TextDrawingMode.TextDrawFillStroke)
        self.assertEqual(gs.blend_mode, BlendMode.BlendXor)
        self.assertEqual(gs.image_blend_mode, BlendMode.BlendXor)
        self.assertEqual(gs.line_cap, LineCap.CapSquare)
        self.assertEqual(gs.line_join, LineJoin.JoinRound)
        self.assertEqual(gs.inner_join, InnerJoin.InnerRound)
        self.assertEqual(gs.miter_limit, 3.14)
        self.assertEqual(gs.inner_miter_limit, 3.14)
        self.assertEqual(gs.master_alpha, 0.42)
        self.assertEqual(gs.line_width, 10.0)
        self.assertEqual(gs.clip_box, box)
        self.assertEqual(gs.line_dash_pattern, dashes)
        self.assertEqual(gs.line_dash_phase, 3.5)
        self.assertIs(gs.stencil, img)

    def test_copy(self):
        box = Rect(0.0, 0.0, 10.0, 20.0)
        dashes = [(1.0, 2.0), (3.0, 4.0)]
        img = Image(np.zeros((10, 10), dtype=np.uint8), PixelFormat.Gray8)
        gs = GraphicsState(
            anti_aliased=True,
            drawing_mode=DrawingMode.DrawEofFill,
            text_drawing_mode=TextDrawingMode.TextDrawFillStroke,
            blend_mode=BlendMode.BlendXor,
            image_blend_mode=BlendMode.BlendXor,
            line_cap=LineCap.CapSquare,
            line_join=LineJoin.JoinRound,
            inner_join=InnerJoin.InnerRound,
            miter_limit=3.14,
            inner_miter_limit=3.14,
            master_alpha=0.42,
            line_width=10.0,
            clip_box=box,
            line_dash_pattern=dashes,
            line_dash_phase=3.5,
            stencil=img
        )
        cpy = gs.copy()

        self.assertTrue(cpy.anti_aliased)
        self.assertEqual(cpy.drawing_mode, DrawingMode.DrawEofFill)
        self.assertEqual(cpy.text_drawing_mode,
                         TextDrawingMode.TextDrawFillStroke)
        self.assertEqual(cpy.blend_mode, BlendMode.BlendXor)
        self.assertEqual(cpy.image_blend_mode, BlendMode.BlendXor)
        self.assertEqual(cpy.line_cap, LineCap.CapSquare)
        self.assertEqual(cpy.line_join, LineJoin.JoinRound)
        self.assertEqual(cpy.inner_join, InnerJoin.InnerRound)
        self.assertEqual(cpy.miter_limit, 3.14)
        self.assertEqual(cpy.inner_miter_limit, 3.14)
        self.assertEqual(cpy.master_alpha, 0.42)
        self.assertEqual(cpy.line_width, 10.0)
        self.assertEqual(cpy.clip_box, box)
        self.assertEqual(cpy.line_dash_pattern, dashes)
        self.assertEqual(cpy.line_dash_phase, 3.5)

        # The image has no comparison operator. Make sure it's a new object
        self.assertIsNot(cpy.stencil, img)
        self.assertFalse(
            array_bases_equal(cpy.stencil.pixels, gs.stencil.pixels)
        )



import unittest
import weakref

import numpy as np
from numpy.testing import assert_equal

import celiagg as agg


class TestCanvas(unittest.TestCase):
    def test_cleanup(self):
        # Make sure our array references don't linger
        buffer = np.zeros((10, 10, 3), dtype=np.uint8)
        canvas = agg.CanvasRGB24(buffer)

        weak_buf = weakref.ref(buffer)
        weak_arr = weakref.ref(canvas.array)
        self.assertIs(weak_buf(), weak_arr())

        del buffer
        self.assertIsNotNone(weak_buf())

        del canvas
        self.assertIsNone(weak_buf())

    def test_bad_method_args(self):
        canvas = agg.CanvasG8(np.zeros((1, 1), dtype=np.uint8))
        pix_format = agg.PixelFormat.Gray8
        gs = agg.GraphicsState()
        path = agg.Path()
        transform = agg.Transform()

        with self.assertRaises(TypeError):
            canvas.draw_image(None, pix_format, transform, gs)
        with self.assertRaises(TypeError):
            canvas.draw_image(canvas.array, None, transform, gs)
        with self.assertRaises(TypeError):
            canvas.draw_image(canvas.array, pix_format, None, gs)
        with self.assertRaises(TypeError):
            canvas.draw_image(canvas.array, pix_format, transform, None)
        # But this version should work
        canvas.draw_image(canvas.image, None, transform, gs)

        with self.assertRaises(TypeError):
            canvas.draw_shape(None, transform, gs)
        with self.assertRaises(TypeError):
            canvas.draw_shape(path, None, gs)
        with self.assertRaises(TypeError):
            canvas.draw_shape(path, transform, None)

        text = 'Hello!'
        font = agg.Font(
            'Times New Roman', 12.0, agg.FontCacheType.RasterFontCache
        )
        with self.assertRaises(TypeError):
            canvas.draw_text(text, None, transform, gs)
        with self.assertRaises(TypeError):
            canvas.draw_text(text, font, None, gs)
        with self.assertRaises(TypeError):
            canvas.draw_text(text, font, transform, None)
        canvas.draw_text(text, font, transform, gs)

    def test_stencil_size_mismatch(self):
        canvas = agg.CanvasRGB24(np.zeros((4, 5, 3), dtype=np.uint8))
        stencil_canvas = agg.CanvasG8(np.zeros((1, 2), dtype=np.uint8))
        gs = agg.GraphicsState(stencil=stencil_canvas.image)
        path = agg.Path()
        transform = agg.Transform()

        with self.assertRaises(agg.AggError):
            canvas.draw_shape(path, transform, gs)

    def test_clear(self):
        expected = np.zeros((4, 4, 3), dtype=np.uint8)
        buffer = np.zeros((4, 4, 3), dtype=np.uint8)
        canvas = agg.CanvasRGB24(buffer)

        expected[:] = 0
        canvas.clear(0, 0, 0)
        assert_equal(expected, canvas.array)

        expected[:] = 128
        canvas.clear(.5, .5, .5)
        assert_equal(expected, canvas.array)

        expected[:] = 255
        canvas.clear(1, 1, 1)
        assert_equal(expected, canvas.array)



import unittest

import numpy as np

from celiagg import Path, ShapeAtPoints


class TestPath(unittest.TestCase):
    def test_sap_bad_init_args(self):
        with self.assertRaises(TypeError):
            ShapeAtPoints(None, [(0, 0), (10, 10)])

    def test_move_to(self):
        pth = Path()
        pth.move_to(10, 10)
        pth.move_to(0, 0)

        self.assertTrue(np.all(pth.vertices() == [[10.0, 10.0], [0.0, 0.0]]))

    def test_copy(self):
        pth = Path()
        pth.move_to(10, 10)
        pth.move_to(0, 0)

        cpy = pth.copy()
        self.assertNotEqual(id(cpy), id(pth))
        self.assertEqual(len(cpy.vertices()), len(pth.vertices()))

        cpy_pts = np.array(list(iter(cpy)))
        pth_pts = np.array(list(iter(pth)))
        self.assertTrue(np.all(cpy_pts == pth_pts))

    def test_reset(self):
        pth = Path()
        pth.move_to(10, 10)
        pth.move_to(0, 0)

        pth.reset()
        self.assertEqual(len(pth.vertices()), 0)

    def test_closing(self):
        pth = Path()
        pth.move_to(10, 10)
        pth.move_to(0, 0)
        pth.close()

        self.assertEqual(len(pth.vertices()), 3)
        self.assertTrue(np.all(pth.vertices()[-1] == [0.0, 0.0]))

    def test_iteration(self):
        pth = Path()
        pth.move_to(10, 10)

        lp = list(iter(pth))
        self.assertEqual(len(lp), 1)
        self.assertTrue(np.all(lp[0] == [10.0, 10.0]))



from __future__ import division

import unittest

import numpy as np
from numpy.testing import assert_equal

import celiagg as agg


class TestDrawing(unittest.TestCase):
    def setUp(self):
        buffer = np.zeros((5, 5), dtype=np.uint8)
        self.canvas = agg.CanvasG8(buffer)
        self.paint = agg.SolidPaint(1/255, 1/255, 1/255)
        self.state = agg.GraphicsState(anti_aliased=False, line_width=1.0)
        self.transform = agg.Transform()

    def test_lines(self):
        path = agg.Path()
        path.move_to(0, .5)
        path.line_to(5, .5)
        expected = [
            [1, 1, 1, 1, 1],
            [0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0],
        ]

        self.canvas.draw_shape(
            path, self.transform, self.state, stroke=self.paint
        )
        assert_equal(expected, self.canvas.array)

        self.canvas.clear(0, 0, 0)
        path.reset()
        path.move_to(.5, 0)
        path.line_to(.5, 5)
        expected = [
            [1, 0, 0, 0, 0],
            [1, 0, 0, 0, 0],
            [1, 0, 0, 0, 0],
            [1, 0, 0, 0, 0],
            [1, 0, 0, 0, 0],
        ]
        self.canvas.draw_shape(
            path, self.transform, self.state, stroke=self.paint
        )
        assert_equal(expected, self.canvas.array)

        self.canvas.clear(0, 0, 0)
        path.reset()
        path.move_to(0, 0)
        path.line_to(5, 5)
        path.move_to(0, 5)
        path.line_to(5, 0)
        self.state.line_width = 0.5
        expected = [
            [1, 0, 0, 0, 1],
            [0, 1, 0, 1, 0],
            [0, 0, 1, 0, 0],
            [0, 1, 0, 1, 0],
            [1, 0, 0, 0, 1],
        ]
        self.canvas.draw_shape(
            path, self.transform, self.state, stroke=self.paint
        )
        assert_equal(expected, self.canvas.array)

    def test_rect(self):
        path = agg.Path()
        path.rect(0, 0, 5, 5)
        expected = [
            [1, 1, 1, 1, 1],
            [1, 0, 0, 0, 1],
            [1, 0, 0, 0, 1],
            [1, 0, 0, 0, 1],
            [1, 1, 1, 1, 1],
        ]
        self.canvas.draw_shape(
            path, self.transform, self.state, stroke=self.paint
        )
        assert_equal(expected, self.canvas.array)

        expected = [
            [1, 1, 1, 1, 1],
            [1, 1, 1, 1, 1],
            [1, 1, 1, 1, 1],
            [1, 1, 1, 1, 1],
            [1, 1, 1, 1, 1],
        ]
        self.canvas.draw_shape(
            path, self.transform, self.state,
            fill=self.paint, stroke=self.paint
        )
        assert_equal(expected, self.canvas.array)

    def test_ellipse(self):
        path = agg.Path()
        path.ellipse(2.5, 2.5, 2, 2)
        expected = [
            [0, 1, 1, 1, 0],
            [1, 1, 0, 1, 1],
            [1, 0, 0, 0, 1],
            [1, 1, 0, 1, 1],
            [0, 1, 1, 1, 0],
        ]
        self.canvas.draw_shape(
            path, self.transform, self.state, stroke=self.paint
        )
        assert_equal(expected, self.canvas.array)



# The MIT License (MIT)
#
# Copyright (c) 2014-2016 WUSTL ZPLAB
# Copyright (c) 2016-2020 Celiagg Contributors
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Authors: Erik Hvatum <ice.rikh@gmail.com>
#          John Wiggins <jwiggins@enthought.com>
import os
import sys

from numpy.distutils.core import setup
from numpy.distutils.misc_util import Configuration

# Disable text rendering with this option
if '--no-text-rendering' in sys.argv:
    del sys.argv[sys.argv.index('--no-text-rendering')]
    os.environ['CELIAGG_TEXT_RENDERING'] = '0'
else:
    os.environ['CELIAGG_TEXT_RENDERING'] = '1'
    os.environ['CELIAGG_USE_PKGCONFIG'] = '1'
    if '--no-freetype-pkg-config' in sys.argv:
        del sys.argv[sys.argv.index('--no-freetype-pkg-config')]
        os.environ['CELIAGG_USE_PKGCONFIG'] = '0'


def configuration(parent_package='', top_path=None):
    config = Configuration(None, parent_package, top_path)
    config.set_options(
        ignore_setup_xxx_py=True,
        assume_default_configuration=True,
        delegate_options_to_subpackages=True,
        quiet=True)

    config.add_subpackage('celiagg')
    return config

with open('README.rst', 'r') as fp:
    long_description = fp.read()

setup(
    name='celiagg',
    configuration=configuration,
    license='MIT',
    version='1.1.1',
    description='Anti-Grain Geometry for Python 3 with Cython',
    long_description=long_description,
    long_description_content_type='text/x-rst',
    url='https://github.com/celiagg/celiagg',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: C++',
        'Programming Language :: Cython',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Operating System :: MacOS',
    ],
    requires=['numpy', 'freetype'],
    packages=['celiagg', 'celiagg.tests']
)



import unittest

from celiagg import (GradientSpread, GradientUnits, SolidPaint,
                     LinearGradientPaint, RadialGradientPaint)

_GRADIENT_STOPS = ((0.0, 1.0, 0.0, 0.0, 1.0),
                   (0.5, 1.0, 1.0, 1.0, 1.0),
                   (1.0, 0.0, 0.0, 0.4, 1.0))


class TestPaint(unittest.TestCase):
    def test_solid_paint(self):
        paint0 = SolidPaint(0.0, 0.0, 0.0)
        paint1 = SolidPaint(0.0, 1.0, 1.0)

        # XXX: There is no `!=` operator for SolidPaint
        self.assertFalse(paint0 == paint1)
        self.assertEqual(paint0.copy(), SolidPaint(0, 0, 0))

    def test_linear_gradient_paint(self):
        paint = LinearGradientPaint(100, 100, 0, 0, _GRADIENT_STOPS,
                                    GradientSpread.SpreadRepeat,
                                    GradientUnits.ObjectBoundingBox)
        paint.copy()

    def test_radial_gradient_paint(self):
        paint = RadialGradientPaint(0, 0, 100, 200, 200, _GRADIENT_STOPS,
                                    GradientSpread.SpreadRepeat,
                                    GradientUnits.UserSpace)
        paint.copy()



import unittest

import numpy as np
from numpy.testing import assert_allclose

from celiagg import Transform


class TestTransform(unittest.TestCase):
    def test_construction(self):
        t = Transform(1.0, 2.0, 3.0, 4.0, 5.0, 6.0)
        self.assertEqual(repr(t), 'Transform(1.0, 2.0, 3.0, 4.0, 5.0, 6.0)')

    def test_copy(self):
        t = Transform(1.0, 2.0, 3.0, 4.0, 5.0, 6.0)
        cpy = t.copy()
        self.assertEqual(repr(t), 'Transform(1.0, 2.0, 3.0, 4.0, 5.0, 6.0)')

    def test_scale(self):
        t = Transform()

        t.scale(2.0, 2.0)
        pt = t.worldToScreen(1.0, 1.0)
        self.assertEqual(pt, (2.0, 2.0))

        t.reset()
        t.scale(1.0, 2.0)
        pt = t.worldToScreen(1.0, 1.0)
        self.assertEqual(pt, (1.0, 2.0))

        t.reset()
        t.scale(2.0, 1.0)
        pt = t.worldToScreen(1.0, 1.0)
        self.assertEqual(pt, (2.0, 1.0))

    def test_rotate(self):
        t = Transform()

        t.rotate(np.pi / 2.0)
        pt = t.worldToScreen(1.0, 0.0)
        assert_allclose(pt, (0.0, 1.0), atol=1e-9)

        t.reset()
        t.rotate(np.pi)
        pt = t.worldToScreen(1.0, 0.0)
        assert_allclose(pt, (-1.0, 0.0), atol=1e-9)

        t.reset()
        t.rotate(-np.pi / 2.0)
        pt = t.worldToScreen(1.0, 0.0)
        assert_allclose(pt, (0.0, -1.0), atol=1e-9)

        t.reset()
        t.rotate(np.pi / 2.0)
        t.rotate(np.pi / 2.0)
        pt = t.worldToScreen(1.0, 0.0)
        assert_allclose(pt, (-1.0, 0.0), atol=1e-9)

    def test_translate(self):
        t = Transform()

        t.translate(10.0, 10.0)
        pt = t.worldToScreen(0.0, 0.0)
        self.assertEqual(pt, (10.0, 10.0))

        t.reset()
        t.translate(-10.0, -10.0)
        pt = t.worldToScreen(0.0, 0.0)
        self.assertEqual(pt, (-10.0, -10.0))

        t.reset()
        t.translate(10.0, 0.0)
        t.translate(0.0, 10.0)
        pt = t.worldToScreen(0.0, 0.0)
        self.assertEqual(pt, (10.0, 10.0))

    def test_roundtrip(self):
        t = Transform()

        t.scale(2.0, 2.0)
        t.rotate(np.pi)
        t.translate(5.0, 5.0)

        pt = t.screenToWorld(*t.worldToScreen(0.0, 0.0))
        assert_allclose(pt, (0.0, 0.0))
        pt = t.screenToWorld(*t.worldToScreen(42.0, 100.0))
        assert_allclose(pt, (42.0, 100.0))



# The MIT License (MIT)
#
# Copyright (c) 2014-2015 WUSTL ZPLAB
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Authors: Erik Hvatum <ice.rikh@gmail.com>
#          John Wiggins <john.wiggins@xfel.eu>

from __future__ import print_function
import os
import os.path as op
import platform
import subprocess
import sys

import numpy
from numpy.distutils.core import setup
from numpy.distutils.misc_util import Configuration

try:
    from Cython.Build import cythonize
except ImportError:
    cythonize = None
    print("Cython does not appear to be installed.  Will attempt to use "
          "pre-made cpp file...")

# These are added to the environment by the main setup.py script
with_text_rendering = os.environ.get('CELIAGG_TEXT_RENDERING', '0') == '1'
with_pkgconfig = os.environ.get('CELIAGG_USE_PKGCONFIG', '0') == '1'


def get_freetype_info():
    """ Use pkg-config to locate the freetype2 library installed on the system.

    If it's not installed at the system level, attempt to find it in the
    pkgconfig directory for the currently running Python interpreter.
    """
    def run_cmd(cmd, env=None):
        with open("/dev/null", "w") as dev_null:
            output = subprocess.check_output(cmd, universal_newlines=True,
                                             env=env, stderr=dev_null.fileno())
        return output.split()

    def collect_data(env=None):
        cmd_prefix = ['pkg-config', 'freetype2']
        commands = {'cflags': ['--cflags'], 'ldflags': ['--libs']}

        data = {}
        try:
            for key, args in commands.items():
                data[key] = run_cmd(cmd_prefix + args, env=env)
        except (subprocess.CalledProcessError, OSError):
            pass
        return data

    data = collect_data()
    if len(data) < 2:
        # Try again with the Python env's pkgconfig directory added
        env = os.environ.copy()
        env['PKG_CONFIG_PATH'] = op.join(sys.exec_prefix, 'lib', 'pkgconfig')
        data = collect_data(env=env)

    if len(data) < 2:
        msg = ("Failed to execute pkg-config freetype2.  If freetype is "
               "installed in standard system locations, it may work to run "
               "this script with --no-freetype-pkg-config.  Otherwise, "
               "appropriate CFLAGS and LDFLAGS environment variables must be "
               "set.\n\n"
               "If you wish to disable text rendering, you can re-run this "
               "script with the --no-text-rendering flag.")
        print(msg, file=sys.stderr)

        # NOTE: Avoid exiting when pip is running an egg_info command. Without
        # this, it's not possible to avoid freetype when installing from pip.
        if 'egg_info' not in sys.argv:
            exit(-1)
        else:
            return [], []
    return data['cflags'], data['ldflags']


def configuration(parent_package='', top_path=None):
    sources = [
        'agg-svn/agg-2.4/src/agg_arc.cpp',
        'agg-svn/agg-2.4/src/agg_arrowhead.cpp',
        'agg-svn/agg-2.4/src/agg_bezier_arc.cpp',
        'agg-svn/agg-2.4/src/agg_bspline.cpp',
        'agg-svn/agg-2.4/src/agg_color_rgba.cpp',
        'agg-svn/agg-2.4/src/agg_curves.cpp',
        'agg-svn/agg-2.4/src/agg_embedded_raster_fonts.cpp',
        'agg-svn/agg-2.4/src/agg_gsv_text.cpp',
        'agg-svn/agg-2.4/src/agg_image_filters.cpp',
        'agg-svn/agg-2.4/src/agg_line_aa_basics.cpp',
        'agg-svn/agg-2.4/src/agg_line_profile_aa.cpp',
        'agg-svn/agg-2.4/src/agg_rounded_rect.cpp',
        'agg-svn/agg-2.4/src/agg_sqrt_tables.cpp',
        'agg-svn/agg-2.4/src/agg_trans_affine.cpp',
        'agg-svn/agg-2.4/src/agg_trans_double_path.cpp',
        'agg-svn/agg-2.4/src/agg_trans_single_path.cpp',
        'agg-svn/agg-2.4/src/agg_trans_warp_magnifier.cpp',
        'agg-svn/agg-2.4/src/agg_vcgen_bspline.cpp',
        'agg-svn/agg-2.4/src/agg_vcgen_contour.cpp',
        'agg-svn/agg-2.4/src/agg_vcgen_dash.cpp',
        'agg-svn/agg-2.4/src/agg_vcgen_markers_term.cpp',
        'agg-svn/agg-2.4/src/agg_vcgen_smooth_poly1.cpp',
        'agg-svn/agg-2.4/src/agg_vcgen_stroke.cpp',
        'agg-svn/agg-2.4/src/agg_vpgen_clip_polygon.cpp',
        'agg-svn/agg-2.4/src/agg_vpgen_clip_polyline.cpp',
        'agg-svn/agg-2.4/src/agg_vpgen_segmentator.cpp',
    ]
    base_path = op.abspath(op.dirname(__file__))
    celiagg_cython_source = op.join(base_path, '_celiagg.pyx')
    celiagg_sources = ['font.cpp', 'glyph_iter.cpp', 'image.cpp', 'paint.cpp',
                       'vertex_source.cpp', '_celiagg.cpp']

    include_dirs = ['agg-svn/agg-2.4/include',
                    'agg-svn/agg-2.4',
                    numpy.get_include()]
    if platform.system() == "Windows":
        # Visual studio does not support/need these
        extra_compile_args = []
    else:
        extra_compile_args = [
           '-Wfatal-errors',
           '-Wno-unused-function',
           '-std=c++11',
        ]
    extra_link_args = []
    define_macros = []

    if with_text_rendering:
        if platform.system() == 'Windows':
            extra_link_args.extend(['Gdi32.lib', 'User32.lib'])
            include_dirs.append('agg-svn/agg-2.4/font_win32_tt')
            font_source = 'agg-svn/agg-2.4/font_win32_tt/agg_font_win32_tt.cpp'
        else:
            if with_pkgconfig:
                cflags, ldflags = get_freetype_info()
                extra_compile_args.extend(cflags)
                extra_link_args.extend(ldflags)
            define_macros.append(('_USE_FREETYPE', 1))
            include_dirs.append('agg-svn/agg-2.4/font_freetype')
            font_source = 'agg-svn/agg-2.4/font_freetype/agg_font_freetype.cpp'
        sources.append(font_source)
        define_macros.append(('_ENABLE_TEXT_RENDERING', 1))

    cython_compile_env = {'_ENABLE_TEXT_RENDERING': with_text_rendering}

    ext_kwargs = {
        'include_dirs': include_dirs,
        'define_macros': define_macros,
        'language':  'c++',
        'extra_compile_args': extra_compile_args,
        'extra_link_args': extra_link_args
    }

    # Installing from an SDIST is special...
    cpp_celiagg = op.join('celiagg', '_celiagg.cpp')
    pyx_celiagg = op.join('celiagg', '_celiagg.pyx')
    is_sdist = op.exists(cpp_celiagg) and not op.exists(pyx_celiagg)

    # Run Cython first if this is a development version
    if not is_sdist and cythonize is not None:
        cythonize(celiagg_cython_source, compile_time_env=cython_compile_env)

    config = Configuration('celiagg', parent_package, top_path)
    config.add_extension('_celiagg', sources=celiagg_sources + sources,
                         **ext_kwargs)
    config.add_subpackage('tests')
    return config


if __name__ == '__main__':
    setup(**(configuration(top_path='').todict()))



from __future__ import unicode_literals

import unittest

import numpy as np

import celiagg as agg


class TestTextDrawing(unittest.TestCase):
    def test_font_byte_string(self):
        canvas = agg.CanvasG8(np.zeros((1, 1), dtype=np.uint8))
        gs = agg.GraphicsState()
        transform = agg.Transform()

        text_unicode = 'Hello!'
        font_unicode = agg.Font(
            'Times New Roman',
            12.0,
            agg.FontCacheType.RasterFontCache,
            face_index=1,
        )
        text_byte = b'Hello!'
        font_byte = agg.Font(
            b'Times New Roman',
            12.0,
            agg.FontCacheType.RasterFontCache,
        )

        canvas.draw_text(text_unicode, font_unicode, transform, gs)
        canvas.draw_text(text_byte, font_unicode, transform, gs)
        canvas.draw_text(text_unicode, font_byte, transform, gs)
        canvas.draw_text(text_byte, font_byte, transform, gs)
